#!/usr/bin/env python
##!/usr/bin/python

import sys
import resource
import time
from subprocess import call
import os

__author__="mforster"
__date__ ="$Jul 11, 2012 01:00 PM$"

def setlimits():
	#############################################################################
	# sets resource limits (mandatory for job interoperability on cluster)
	# to 10MB Stack + 1GB data + 1GB RLIMIT_RSS
	# first steps taken from
	# http://docs.python.org/library/resource.html and
	# http://www.doughellmann.com/PyMOTW/resource/
	# Author: MICHAEL FORSTER m.forster@ikmb.uni-kiel.de
	# Version: 13 September 2010
	#############################################################################

	#############################################################################
	# Adjust the maximum size (in bytes) of the process's heap
	#############################################################################

	soft, hard = resource.getrlimit(resource.RLIMIT_DATA)
	#print 'Data/heap limit starts as  :', soft

	# 1GB heap:
	newheap=1024*1024*1024

	resource.setrlimit(resource.RLIMIT_DATA, (newheap, hard))

	soft, hard = resource.getrlimit(resource.RLIMIT_DATA)
	#print 'Data/heap limit changed to :', soft

	#############################################################################
	# ca 10MB original stack changed to exactly 10MB stack:
	#############################################################################

	#newstack=10*1024*1024+1
	newstack=10*1024*1024
	soft, hard = resource.getrlimit(resource.RLIMIT_STACK)
	#print 'Stack limit starts as  :', soft
	resource.setrlimit(resource.RLIMIT_STACK, (newstack, hard))
	soft, hard = resource.getrlimit(resource.RLIMIT_STACK)
	#print 'Stack limit changed to :', soft

	#############################################################################
	# 1GB RLIMIT_RSS
	# "Maximale resident set size in Byte. Bei Speicherengpaessen entzieht der Kern
	# den Prozessen, die den resident set size ueberschreiten, den zuviel
	# angeforderten Speicher."
	# http://pronix.linuxdelta.de/C/Linuxprogrammierung/Linuxsystemprogrammieren_C_Kurs_Kapitel3a.shtml
	#############################################################################

	newrss=1024*1024*1024
	soft, hard = resource.getrlimit(resource.RLIMIT_RSS)
	#print 'RLIMIT_RSS starts as  :', soft
	resource.setrlimit(resource.RLIMIT_RSS, (newrss, hard))
	soft, hard = resource.getrlimit(resource.RLIMIT_RSS)
	#print 'RLIMIT_RSS changed to :', soft


def pib_chrm_to_crs_refbase_diffs ():
	#######################################################################################
	# Define refbase differences in Anderson compared to chrM
	#######################################################################################
	crsbase = {}
	crsbase['73']='A'
	crsbase['150']='C'
	crsbase['195']='T'
	crsbase['263']='A'
	crsbase['309.1']='-'
	crsbase['310']='T'
	crsbase['315.1']='-'
	crsbase['408']='T'
	crsbase['15932']='T'
	crsbase['16172']='T'
	crsbase['16183']='A'
	crsbase['16189']='T'
	crsbase['16193.1']='-'
	crsbase['16223']='C'
	crsbase['16320']='C'
	crsbase['16519']='T'
	return crsbase

def pib_chrm_to_crs_posref (h, hg18ref, crsbase):
	#######################################################################################
	# Compute CRS-based position (as text string!) and ref base
	# 
	# h       : hg18 based position (hg18pos)
	# hg18ref : reference base in hg18-chrM
	# crsbase : if not empty, then the CRS reference base
	#######################################################################################

	### get CRS position: ###
	
	n = int(h)
	if n <= 309:
		c = '%s' % (n)
	elif n == 310:
		c = '309.1'
	elif n <= 316:
		c = '%s' % (n-1)
	elif n == 317:
		c = '315.1'
	elif n <= 3108:
		c = '%s' % (n-2)
	elif n <= 16194:
		c = '%s' % (n-1)
	elif n == 16195:
		c = '16193.1'
	else:
		c = '%s' % str(n-2)
	
	### get CRS refbase: ###
	
	if c in crsbase:
		crsref = crsbase[c]
	else:
		crsref = hg18ref
	crspos = c
	return crspos, crsref
	

def pib_chrm_to_crs_line (cols, ic, crspos, crsref):
	#######################################################################################
	# Translate pibase_consensus imput line to CRS pos and ref
	#
	# cols    : columns of current line from pibase_consensus input file
	# ic[0]   : column with hg18-coord
	# ic[1]   : column with hg18-refbase
	# crspos  : CRS coord
	# crsref  : CRS refbase
	#######################################################################################
	
	cols[ ic[0] ] = crspos
	cols[ ic[1] ] = crsref
	
	line = ''
	n = len(cols)
	for i in range(0,n-1):
		line += cols[i] + '\t'
		
	line += cols[n-1] + '\n'
	return line

def pib_chrm_to_crs_alleles (alleles):
	#############################################################################
	# Find major and minor alleles from allele counts, and compute coverage
	# 
	# Argument:
	# alleles : ACGT read counts string, e.g. 0:30:0:15
	# 
	# Returned results:
	# a       : major allele 
	# b       : minor allele 
	# c       : coverage (excluding Ns) 
	# amax    : read counts supporting the major allele 
	# bmax    : read counts supporting the minor allele 
	#############################################################################
	
	# counts per allele
	covs = alleles.split(':')
	
	# dictionary for translating covs index --> nucleotide
	bases = {0: 'A', 1: 'C', 2: 'G', 3: 'T'}
	
	# translate allelic coverages character into integer and get total coverage
	cov = []
	c = 0
	for i in range(0,4):
		j = int(covs[i])
		cov.append(j)
		c += j
		
	# find major allele
	amax = -1
	ia = -1
	for i in range(0,4):
		if cov[i] > max(0,amax):
			ia = i
			amax = cov[i]
			
	# find minor allele
	bmax = -1
	ib = -1
	for i in range(0,4):
		if i == ia:
			continue
		elif cov[i] > max(0,bmax):
			ib = i
			bmax = cov[i]
	
	# major allele:
	a=''
	if ia in bases:
		a = bases[ia]
	
	# major allele:
	b=''
	if ib in bases:
		b = bases[ib]
		
	return a, b, amax, bmax, c

def pib_chrm_to_crs_variant (t, minr, alleles, fmaj1, fmaj2, fmin1, fmin2, outa, outb, cols, ic, crspos, crsref):
	#######################################################################################
	# if variant with respect to CRS: 
	# add variant and allele counts/percentage to list, and output line to file.
	#
	# Arguments:
	# t       : min coverage (fraction of coverage, e.g. 0.01) for minor allele
	# minr    : min coverage (absolute) for allele
	# alleles : ACGT read counts string, e.g. 0:30:0:15
	# fmaj1   : list of tuples for major mtdna type HVS-1 (coord and variant, counts, percentages)
	# fmaj2   : list of tuples for major mtdna type HVS-2 (coord and variant, counts, percentages)
	# fmin1   : list of tuples for minor mtdna type HVS-1 (coord and variant, counts, percentages)
	# fmin2   : list of tuples for minor mtdna type HVS-2 (coord and variant, counts, percentages)
	# outa    : handle of major allele file (pibase_consensus format, CRS variants only)
	# outb    : handle of minor allele file (pibase_consensus format, CRS variants only)
	# cols    : columns of current line from pibase_consensus input file
	# ic[0]   : column with hg18-coord
	# ic[1]   : column with hg18-refbase
	# crspos  : CRS coord
	# crsref  : CRS refbase
	#
	# Returned Results:
	# fmaj    : list of tuples for major mtdna type (coord and variant, counts, percentages)
	# fmin    : list of tuples for minor mtdna type (coord and variant, counts, percentages)
	#######################################################################################
	
	# find major and minor alleles from allele counts, and compute coverage:
	a, b, ac, bc, c = pib_chrm_to_crs_alleles (alleles)
	
	# check whether coverage:
	if c == 0:
		return fmaj1, fmaj2, fmin1, fmin2
	
	# Minor allele: check whether minor allele coverage is greater than thresholds:
	if bc >= max(minr, t*c):
		# if variant with respect to CRS:
		# add variant and allele counts/percentage to list, and output line to file.
		fmin1, fmin2 = pib_chrm_to_crs_variant_sub (t, minr, b, bc, c, fmin1, fmin2, outb, cols, ic, crspos, crsref)

	# Major allele: check whether major allele coverage is greater than threshold:
	if ac >= minr:
		# if variant with respect to CRS:
		# add variant and allele counts/percentage to list, and output line to file.
		fmaj1, fmaj2 = pib_chrm_to_crs_variant_sub (t, minr, a, ac, c, fmaj1, fmaj2, outa, cols, ic, crspos, crsref)

	return fmaj1, fmaj2, fmin1, fmin2

def pib_chrm_to_crs_variant_sub (t, minr, a, ac, c, f1, f2, out, cols, ic, crspos, crsref):
	#######################################################################################
	# for pib_chrm_to_crs_variant: if variant with respect to CRS: 
	# add variant and allele counts/percentage to list, and output line to file.
	#
	# Arguments:
	# t       : min coverage (fraction of coverage, e.g. 0.01) for minor allele
	# minr    : min coverage (absolute) for allele
	# a       : allele, e.g. A
	# ac      : allele coverage
	# c       : total coverage excluding N
	# f1      : list of tuples for mtdna type HVS-1 (coord and variant, counts, percentages)
	# f2      : list of tuples for mtdna type HVS-2 (coord and variant, counts, percentages)
	# outa    : handle of major allele file (pibase_consensus format, CRS variants only)
	# outb    : handle of minor allele file (pibase_consensus format, CRS variants only)
	# cols    : columns of current line from pibase_consensus input file
	# ic[0]   : column with hg18-coord
	# ic[1]   : column with hg18-refbase
	# crspos  : CRS coord
	# crsref  : CRS refbase
	#
	# Returned Results:
	# f1      : list of tuples for mtdna type (coord and variant, counts, percentages)
	#######################################################################################

	# check whether allele is a variant with respect to CRS:
	if a != crsref:
		# build coord and variant string for allele:
		k = crspos.find('.')
		if k > -1 :
			s = crspos + a
			n = int(crspos[:k])
		else:
			s = crspos + '.' + a
			n = int(crspos)
		
		# compute allele coverage as percentage of coverage:
		pc = 100 * float(ac) / float(c) + 0.5
		ipc = int (pc)
	
		# append CRS variant and coverage counts and percentage to list as tuple:
		if n < 500:
			f2.append( (s, ac, ipc) )
		elif n > 16050:
			f1.append( (s, ac, ipc) )
	
		# translate line to CRS pos and ref:
		line1 = pib_chrm_to_crs_line (cols, ic, crspos, crsref)
	
		# write line to outa:
		out.write(line1)
	return f1, f2

def pib_chrm_to_crs_cleanup (f4maj1, f4maj2, f4min1, f4min2, f0maj1, f0maj2, f0min1, f0min2):
	#######################################################################################
	# Remove alignment artifacts in HVS-I
	#######################################################################################
	
	# most stringent filter (f4) major mtdna type:
	f4maj1 = pib_chrm_to_crs_cleanup_sub (f4maj1)
	
	# least stringent filter (f0) major mtdna type:
	f0maj1 = pib_chrm_to_crs_cleanup_sub (f0maj1)
	
	# most stringent filter (f4) minor mtdna type:
	f4min1 = pib_chrm_to_crs_cleanup_sub (f4min1)
	
	# least stringent filter (f0) minor mtdna type:
	f0min1 = pib_chrm_to_crs_cleanup_sub (f0min1)

	# if f4-variant not in f0: remove this variant from f4:
	f4maj1 = pib_chrm_to_crs_cleanup_del (f4maj1, f0maj1)
	f4maj2 = pib_chrm_to_crs_cleanup_del (f4maj2, f0maj2)
	f4min1 = pib_chrm_to_crs_cleanup_del (f4min1, f0min1)
	f4min2 = pib_chrm_to_crs_cleanup_del (f4min2, f0min2)
	
	return f4maj1, f4maj2, f4min1, f4min2, f0maj1, f0min1
	
def pib_chrm_to_crs_cleanup_sub (f1):
	#######################################################################################
	# Remove alignment artifacts in HVS-I
	#######################################################################################
	
	fnew = []
	n1 = len(f1)
	
	arti=0
	h=0
	for i in range(0,n1):

		# get coordinate.variant, read counts, read percentage for this coordinate:
		s, ac, ipc = f1[i]
		# check for special cases: 
		# 1) 16189.C,16190.T
		# 2) 16193.1C if 16189.T
		# and discard each of these as an alignment artifact
		# with respect to the African chrM reference sequence:
		if arti == 1:
			# this is a 16190.T
			arti = 0
		elif s == '16189.C' :
			try:
				s1, ac1, ipc1 = f1[i+1]
				if s1 == '16190.T':
					arti=1
					h=1
			except:
				s1='dummy'
		else:
			if s == '16189.T':
				# remember this for the next "if" below:
				h=1
			if s == '16193.1C' and h == 1:
				# skip this unlikely C-insertion
				continue
			fnew.append((s, ac, ipc))
	return fnew

def pib_chrm_to_crs_cleanup_del (f4, f0):
	#######################################################################################
	# if f4-variant not in f0: remove this variant from f4
	#######################################################################################

	#### identify and mark sequence variants in f4 which are not in f0 ####
	n4 = len(f4)
	n0 = len(f0)
	dels = []
	for i in range(0,n4):
		s4, ac4, ipc4 = f4[i]
		found=0
		for j in range(0,n0):
			s0, ac0, ipc0 = f0[j]
			if s0 == s4:
				found=1
				break
		if found == 0:
			dels.append(i)
	#### deleted the marked sequence variants from f4 ####
	n = len(dels)
	k = 0
	for i in range(0,n):
		j = i - k
		try:
			del f4[j]
		except:
			print 'n, j, k', n, j, k 
		k += 1
	return f4

def pib_chrm_to_crs_summary (out, f4maj1, f4maj2, f4min1, f4min2, f0maj1, 
	f0maj2, f0min1, f0min2, owner, project, sample):
	#######################################################################################
	# Output summary file
	#######################################################################################

	# most stringent filter (f4) major mtdna type:
	stub = 'f4maj'
	pib_chrm_to_crs_summary_sub (out, f4maj1, f4maj2, stub, owner, project, sample)
	
	# least stringent filter (f0) major mtdna type:
	stub = 'f0maj'
	pib_chrm_to_crs_summary_sub (out, f0maj1, f0maj2, stub, owner, project, sample)
	
	# most stringent filter (f4) minor mtdna type:
	stub = 'f4min'
	pib_chrm_to_crs_summary_sub (out, f4min1, f4min2, stub, owner, project, sample)
	
	# least stringent filter (f0) minor mtdna type:
	stub = 'f0min'
	pib_chrm_to_crs_summary_sub (out, f0min1, f0min2, stub, owner, project, sample)

def pib_chrm_to_crs_summary_sub (out, f1, f2, stub, owner, project, sample):
	#######################################################################################
	# Output summary file
	#######################################################################################

	n1 = len(f1)
	n2 = len(f2)
	line = owner + '\t' + project + '\t' + sample + stub + '\t?\t1\t'
	line1 = '#Reads_' + owner + '\t' + project + '\t' + sample + stub + '\t?\t1\t'
	line2 = '#Percent_' + owner + '\t' + project + '\t' + sample + stub + '\t?\t1\t'
	
	# HVS-I :
	arti=0
	h=0
	for i in range(0,n1):
		# get coordinate.variant, read counts, read percentage for this coordinate:
		s, ac, ipc = f1[i]
		# append coordinate.variant:
		line += s + '\t'
		# append reads:
		p = '%s\t' % (ac)
		line1 += p
		# append percentage of coverage:
		p = '%s\t' % (ipc)
		line2 += p

	# pad with tabs
	for i in range(n1,22):
		line += '\t'
		line1 += '\t'
		line2 += '\t'

	# HVS-II :
	for i in range(0,n2):
		# get coordinate.variant, read counts, read percentage for this coordinate:
		s, ac, ipc = f2[i]
		# append coordinate.variant:
		line += s + '\t'
		# append reads:
		p = '%s\t' % (ac)
		line1 += p
		# append percentage of coverage:
		p = '%s\t' % (ipc)
		line2 += p

	# output lines:
	out.write ('#\n')
	out.write (line1 + '\n')
	out.write (line2 + '\n')
	out.write (line + '\n')


def pib_chrm_to_crs (infile1, outbase, owner, project, sample, t, minr):
	'''        #######################################################################################
	#  pibase_chrm_to_crs v1.4.3
	#  Extracts CRS (Anderson 1981) variants from hg18-chrM file (chr# MUST be ONLY mtdna!)
	#######################################################################################
	#  pibase_chrm_to_crs  <in>  <out>  <own>  <pro>  <sam>  [[t] [min]]
	#  
	#  <in>  pibase_consensus genotype file (MUST be chrM ONLY! chr# is not checked!)
	#  <out> base name of output files. The following files are output:
	#        <out>_f4maj.txt: major variants for highest-stringency filter (f4) 
	#        <out>_f4min.txt: minor variants for highest-stringency filter (f4) 
	#        <out>_f0maj.txt: major variants for lowest-stringency filter (f0)
	#        <out>_f0min.txt: minor variants for lowest-stringency filter (f0)
	#        <out>_all.txt: complete file in CRS (Cambridge Reference Sequence) notation
	#        <out>_sum.txt: summary file in CRS notation
	#  <own> for summary: abbreviation of owner/PrincipalInvestigator of the project
	#  <pro> for summary: project abbreviation
	#  <sam> for summary: sample abbreviation
	#
	#  optional arguments:
	#  [t]   threshold for minor allele cut-off 
	#        (default: 0.01, i.e. 1% of total read counts at a given coordinate)
	#  [min] min number of read counts required for an allele-call (default: 1) 
	#######################################################################################
	#  Author: Michael Forster, m.forster@ikmb.uni-kiel.de
	#          in cooperation with Peter Forster, pf223@cam.ac.uk
	#  Date:   11-13 July 2011, 15 July 2011, 18-20 July 2011
	#######################################################################################
	'''
	
	outfilef4a = outbase + '_f4maj.txt'
	outfilef4b = outbase + '_f4min.txt'
	outfilef0a = outbase + '_f0maj.txt'
	outfilef0b = outbase + '_f0min.txt'
	outfileall = outbase + '_all.txt'
	outfile = outbase + '_sum.txt'
	print 'input file  : ', infile1
	print 'output files: ', outfilef4a, outfilef4b, outfilef0a, outfilef0b, outfileall, outfile
	print 'project owner       : ', owner
	print 'project abbreviation: ', project
	print 'sample abbreviation : ', sample
	print 'options: minor allele threshold >= %s of total reads, min reads for allele call >= %s' % (t, minr)

	#######################################################################################
	# Initialise output file header lines
	#######################################################################################

	headers=[]
	
	datstr = time.strftime("%Y_%m_%d__%H_%M_%S")
	outtime = datstr + "\n"
	headers.append ('# File created by pibase_chrm_to_crs v1.4.3 on '+ outtime)
	headers.append ('# from file: '+infile1+'\n')
	headers.append ('# Options: minor allele threshold >= %s of total reads, min reads for allele call >= %s\n#\n' % (t, minr) )

	#######################################################################################
	# Open input file and read table header lines
	# find columns of position, refbase, coverage counts
	#######################################################################################
	
	inf1 = open(infile1, 'r')
	
	# find table header of input file
	while True :
		line = inf1.readline()
		if len(line) == 0:
			# EOF
			sys.exit (' *** Error: End of file %s reached before table header could be found' % (infile1))
		if line.startswith('# C\tPos[1]\tRef\tRefSeqContext\tGC[%]\tClass\tIgn\tBestGen\tBestQual'):
			header = line
			line1 = line.replace('RefSeq','hg18-RefSeq')
			line1 = line1.replace('Ref\t','CRS-Ref\t')
			headers.append(line1)
			break
		else:
			headers.append(line)

	############################################################################
	# find columns pos, ref, first CV, last CV
	############################################################################
	
	header = header.strip()
	# print header
	cols=header.split('\t')
	c=[]
	# ic[0], ic[1]:
	c.append('Pos[1]')
	c.append('Ref')
	# ic[2]:
	c.append('CV')

	ic=[]
	maxcol=0
	for i in c:
		# print i
		n=0
		for col in cols:
			if i==col:
				# print 'found '+col+' in column '+str(n)
				ic.append(n)
				maxcol=max(maxcol,n)
				break
			n+=1
	if len(ic) != len(c):
			print ' *** Error: 1 or more columns of interest not found in header line'
			print  c
			print ic
			print ' *** header line = ' + header
			sys.exit (' *** Aborted' )
	#print 'ic',ic
	
	# modify first CV column (== genotype) to first CV-counts column:
	ic[2] = ic[2]+10
	# get last CV column --> ic[3]:
	ic.append(ic[2]+8)
		
	#######################################################################################
	# Open output files and write header lines
	#######################################################################################
	
	out=open(outfile,'w')
	for i in range(0,3):
		out.write (headers[i])
	
	out4a=open(outfilef4a,'w')
	for h in headers:
		out4a.write (h)
	
	out4b=open(outfilef4b,'w')
	for h in headers:
		out4b.write (h)
	
	out0a=open(outfilef0a,'w')
	for h in headers:
		out0a.write (h)
	
	out0b=open(outfilef0b,'w')
	for h in headers:
		out0b.write (h)
	
	outa=open(outfileall,'w')
	for h in headers:
		outa.write (h)
	
	#######################################################################################
	# Define refbase differences in Anderson compared to chrM
	#######################################################################################
	
	crsbase = pib_chrm_to_crs_refbase_diffs()

	#######################################################################################
	# initialise lists of major and minor mtDNA types for f4 and f0 coverage stringencies 
	#######################################################################################
	
	# filter4 CRS HVS-1 and HVS-2 variant list of tuples: 
	# major allele, minor allele, read counts major & minor, read percentage major & minor 
	f4maj1 =[]
	f4min1 =[]
	f4maj2 =[]
	f4min2 =[]
	
	# filter0 CRS variant list of tuples: 
	# major allele, minor allele, read counts major & minor, read percentage major & minor 
	f0maj1 =[]
	f0min1 =[]
	f0maj2 =[]
	f0min2 =[]
	
	#######################################################################################
	# Initialise start/end coords of considered HVS-I and HVS-II ranges
	#######################################################################################
	
	hvstart=[20000,20000]
	hvend=[-1,-1]
	hvdone=[0,0]
	
	#######################################################################################
	# Loop over all lines in input file
	#######################################################################################

	hv=0
	while True :
		line = inf1.readline()
		if len(line) == 0:
			# EOF
			break
		cols = line.split('\t')

		#print line

		# get hg18-chrM-based position and refbase:
		hg18pos = cols[ic[0]]
		hg18ref = cols[ic[1]]
		
		# compute CRS-based position (as text string!) and ref base:
		crspos, crsref = pib_chrm_to_crs_posref (hg18pos, hg18ref, crsbase)
		
		# update considered HVS-I and HVS-II ranges:
		icrspos = int(crspos.split('.')[0])
		if icrspos < 500:
			if hvdone[hv]==0:
				hvstart[hv] = icrspos
				hvend[hv] = icrspos
				hvdone[hv] = 1
			else:
				hvend[hv] = icrspos
		elif icrspos > 16050:
			hv=1
			if hvdone[hv]==0:
				hvstart[hv] = icrspos
				hvend[hv] = icrspos
				hvdone[hv] = 1
			else:
				hvend[hv] = icrspos
		
		# get allelic read counts for filter4 and filter0
		allelesf4 = cols[ic[2]]
		allelesf0 = cols[ic[3]]

		# translate line to CRS pos and ref:
		line1 = pib_chrm_to_crs_line (cols, ic, crspos, crsref)

		# write line to outa:
		outa.write(line1)
		
		# if variant with respect to CRS: 
		# add variant and allele counts/percentage to list, and output line to file:
		
		#print 'step 1'
		f4maj1, f4maj2, f4min1, f4min2 = pib_chrm_to_crs_variant (t, minr, allelesf4, 
		 f4maj1, f4maj2, f4min1, f4min2, out4a, out4b, cols, ic, crspos, crsref)
		 
		#print 'step 2'
		f0maj1, f0maj2, f0min2, f0min2 = pib_chrm_to_crs_variant (t, minr, allelesf0, 
		 f0maj1, f0maj2, f0min1, f0min2, out0a, out0b, cols, ic, crspos, crsref)
		
	#######################################################################################
	# Remove alignment artifacts
	#######################################################################################

	f4maj1, f4maj2, f4min1, f4min2, f0maj1, f0min1 = pib_chrm_to_crs_cleanup \
	(f4maj1, f4maj2, f4min1, f4min2, f0maj1, f0maj2, f0min1, f0min2)
	
	#######################################################################################
	# Output summary file
	#######################################################################################
	
	#print f4maj1
	#print f4maj2
	#print f4min1
	#print f4min2
	#print f0maj1
	#print f0maj2
	#print f0min1
	#print f0min2
	if hvstart[1] != 20000:
		out.write('# HVS-I considered from np %s to np %s\n' % (hvstart[1], hvend[1]))
	if hvstart[0] != 20000:
		out.write('# HVS-II considered from np %s to np %s\n' % (hvstart[0], hvend[0]))
	
	if hvstart[0] != 20000 or hvstart[1] != 20000:
		pib_chrm_to_crs_summary (out, f4maj1, f4maj2, f4min1, f4min2, f0maj1, f0maj2, f0min1, f0min2, owner, project, sample)
	else:
		out.write('# ***** WARNING: No coordinates were specified in HVS-I or HVS-II ***** \n')
		
	#######################################################################################
	# close all files
	#######################################################################################
	
	inf1.close()
	out.close()
	outa.close()
	out4a.close()
	out4b.close()
	out0a.close()
	out0b.close()

##############################################
# the actual main
##############################################

n = len(sys.argv) 
if n < 6:
	print pib_chrm_to_crs.__doc__
	sys.exit()
else:
	infile = sys.argv[1]
	outbase = sys.argv[2]
	owner = sys.argv[3]
	project = sys.argv[4]
	sample = sys.argv[5]
	t = 0.01
	minr = 1
	try:
		if n > 6:
			t = float(sys.argv[6])
		if n > 7:
			minr = int(sys.argv[7])
	except:
		print pib_chrm_to_crs.__doc__
		sys.exit()
	
setlimits()

pib_chrm_to_crs (infile, outbase, owner, project, sample, t, minr)
print('Done')
